//
//  IFS3D.h
//  IFS3D
//
//  Copyright (c) 2001 David Leppik.
//

#import <Foundation/Foundation.h>

#import "IFS3DDefaults.h"
#import "Shape.h"

#define randomPositiveFloat() 	(((float) (random()))/((float) LONG_MAX))
#define randomUnitFloat()  	(((float) (random()))/((float) LONG_MAX/2)-1)
#define randomFloat(min,max)    (  randomPositiveFloat()*(max-min)+min  )


typedef	float	POINT[ 3 ];
typedef	float	VECTOR[ 3 ];

struct branch {
    VECTOR translation;
    VECTOR rotation;
    VECTOR scale;
};


/** The fractal itself. */
@interface IFS3D : NSObject {
    id	lastView;
}

//------------------------------------------------------------------------------------
//  Update
//------------------------------------------------------------------------------------

- (void)updateBranches;

//------------------------------------------------------------------------------------
//  Drawing
//------------------------------------------------------------------------------------

- (void)draw3DWithView:(id) view;

//------------------------------------------------------------------------------------
//  Getters and setters
//------------------------------------------------------------------------------------

/** If true, fractal is limited to 2 dimensions */
- (BOOL)isFlat;
- (void)setIsFlat:(BOOL)newIsFlat;

/** Recursive depth of the fractal */
- (int)iterations;
- (void)setIterations:(int) newIterations;

/** Number of branches, a.k.a. complexity */
- (int)branchCount;
- (void)setBranchCount:(int) newBranchCount;

/** speed of updates */
- (float)speed;
- (void)setSpeed:(float) newSpeed;

/** The shape drawn at the recursion tree's leaves */
-(NSObject <Shape> *)shape;
-(void)setShape:(NSObject <Shape> *)newShape;

@end
