//
//  IFS3D.m
//  IFS3D
//
//  Copyright (c) 2001 David Leppik.
//

#import <ScreenSaver/ScreenSaver.h>

#import "IFS3D.h"

#import "ColorTriangle.h"
#import "PointShape.h"
#import "Raindrop.h"

@implementation IFS3D

BOOL		isFlat;
int 		iterations;
int		branchCount;  // UI calls this complexity
float           speed;
NSObject <Shape>	*shape;


struct branch	branches[MAX_BRANCHES];
struct branch	velocity[MAX_BRANCHES];
    
struct branch	maxBranch; // highest values
struct branch	minBranch; // lowest values
struct branch	maxDeltaV; // maximum change in velocity per frame
float 		boundaryElasticity; // -1 to 0; change in velocity when limit is hit

//------------------------------------------------------------------------------------
//  Initialization
//------------------------------------------------------------------------------------

- (id)init {
    int i, dim;
    
    self = [super init];
    if (self) {
        NSDictionary * fd = [IFS3DDefaults factoryDefaults];
        [self setIsFlat:[(NSNumber *) [fd objectForKey:@"fractal.isFlat"] boolValue]];
        [self setIterations:[(NSNumber *) [fd objectForKey:@"fractal.iterations"] intValue]];
        [self setBranchCount:[(NSNumber *) [fd objectForKey:@"fractal.branchCount"] intValue]];
        [self setShape:[IFS3DDefaults 
                            shapeForName:(NSString *) [fd objectForKey:@"fractal.shape"]]];
	[self setSpeed:[(NSNumber *) [fd objectForKey:@"fractal.speed"] floatValue]];

        srandom( time( 0 ));
                
        boundaryElasticity = -0.8;
        for (dim=0; dim<3;dim++) {
            minBranch.translation[dim]	= -2.0;
            minBranch.rotation[dim]	=-90.0;
            minBranch.scale[dim]	= 0.05; 
            
            maxBranch.translation[dim]	=  2.0;
            maxBranch.rotation[dim]	= 90.0; 
            maxBranch.scale[dim]	=  1.0;

            
            maxDeltaV.translation[dim]	=  0.00001 * speed;
            maxDeltaV.rotation[dim]	=  0.000001 * speed;
            maxDeltaV.scale[dim]	=  0.000001 * speed;
        }
        
        // Initialize branches & velocity
        // we init all possible branches so we won't have to init if the count increases
        for (i=0; i<MAX_BRANCHES; i++) {
            for (dim=0; dim<3;dim++) {
                branches[i].translation[dim]	= randomFloat(minBranch.translation[dim],maxBranch.translation[dim]);
                branches[i].rotation[dim]	= randomFloat(minBranch.rotation[dim],maxBranch.rotation[dim]);
                branches[i].scale[dim]		= randomFloat(minBranch.scale[dim],maxBranch.scale[dim]);
                
                velocity[i].translation[dim]	= randomUnitFloat()*maxDeltaV.translation[dim];
                velocity[i].rotation[dim]	= randomUnitFloat()*maxDeltaV.rotation[dim];
                velocity[i].scale[dim]		= randomUnitFloat()*maxDeltaV.scale[dim];
            }
        }
    }
        
    return self;

}

//------------------------------------------------------------------------------------
//  Update
//------------------------------------------------------------------------------------

- (void)updateBranches
{
    int i, dim;
    for (i=0; i<branchCount; i++) {
        for (dim=0; dim<3;dim++) {        
            velocity[i].translation[dim]	+= randomUnitFloat()*maxDeltaV.translation[dim];
            velocity[i].rotation[dim]		+= randomUnitFloat()*maxDeltaV.rotation[dim];
            velocity[i].scale[dim]		+= randomUnitFloat()*maxDeltaV.scale[dim];
            
            // Reverse velocitity & slow down if they get too great
            
            if (   velocity[i].translation[dim] + branches[i].translation[dim] > maxBranch.translation[dim]
                || velocity[i].translation[dim] + branches[i].translation[dim] < minBranch.translation[dim]) {
                velocity[i].translation[dim] *=  boundaryElasticity;
                /*
                printf("branch %d\tdim %d\tTRANS %f, %f\trot %f, %f\tscale %f, %f\n", 
                        i, dim ,
                        branches[i].translation[dim], velocity[i].translation[dim],
                        branches[i].rotation[dim], velocity[i].rotation[dim],
                        branches[i].scale[dim], velocity[i].scale[dim]);
                */
                
                //NSBeep();
                
            }
            if (   velocity[i].rotation[dim] + branches[i].rotation[dim] > maxBranch.rotation[dim]
                || velocity[i].rotation[dim] + branches[i].rotation[dim] < minBranch.rotation[dim]) {
                velocity[i].rotation[dim] *=  boundaryElasticity;
                /*
                printf("branch %d\tdim %d\\ttrans %f, %f\tROT %f, %f\tscale %f, %f\n", 
                        i, dim ,
                        branches[i].translation[dim], velocity[i].translation[dim],
                        branches[i].rotation[dim], velocity[i].rotation[dim],
                        branches[i].scale[dim], velocity[i].scale[dim]);
                */
                //NSBeep();
            }
            if (   velocity[i].scale[dim] + branches[i].scale[dim] > maxBranch.scale[dim]
                || velocity[i].scale[dim] + branches[i].scale[dim] < minBranch.scale[dim]) {
                velocity[i].scale[dim] *=  boundaryElasticity;
                /*
                printf("branch %d\tdim %d\ttrans %f, %f\trot %f, %f\tSCALE %f, %f\n", 
                        i, dim ,
                        branches[i].translation[dim], velocity[i].translation[dim],
                        branches[i].rotation[dim], velocity[i].rotation[dim],
                        branches[i].scale[dim], velocity[i].scale[dim]);
                */
                //NSBeep();
            }
            
            branches[i].translation[dim]	+= velocity[i].translation[dim];
            branches[i].rotation[dim]		+= velocity[i].rotation[dim];
            branches[i].scale[dim]		+= velocity[i].scale[dim];
            
            //branches[i].scale[dim] = maxBranch.scale[dim];
                /*
                printf("branch %d\tdim %d\ttrans %f, %f\trot %f, %f\tSCALE %f, %f\n", 
                        i, dim ,
                        branches[i].translation[dim], velocity[i].translation[dim],
                        branches[i].rotation[dim], velocity[i].rotation[dim],
                        branches[i].scale[dim], velocity[i].scale[dim]);
                */
        }
    }
}

//------------------------------------------------------------------------------------
//  Drawing
//------------------------------------------------------------------------------------

- (void)draw3DWithDepth:(int)depth
{
    int i;
    if (depth > 0) {
        for( i=0; i<branchCount; i++) {
            struct branch b = branches[i];
            glPushMatrix();
            if (isFlat) {
                glTranslatef( b.translation[0], 0,  b.translation[2] );
                glScalef( b.scale[0], 1, b.scale[1] );
                glRotatef( b.rotation[1], 0.0, 1.0, 0.0 );
            }
            else {
                glTranslatef( b.translation[0], b.translation[1], b.translation[2] );
                glScalef( b.scale[0], b.scale[1], b.scale[2] );
                glRotatef( b.rotation[0], 1.0, 0.0, 0.0 );
                glRotatef( b.rotation[1], 0.0, 1.0, 0.0 );
                glRotatef( b.rotation[2], 0.0, 0.0, 1.0 );
            }
            [self draw3DWithDepth:depth-1];
            glPopMatrix();
        }
    }
    else {
        [shape draw3D];
    }
}

- (void)draw3DWithView:(id)view
{
    if (view != lastView) {
        [shape setView:view];
        lastView = view;
    }
    [shape prepareToDraw];
    [self draw3DWithDepth:iterations];
}

//------------------------------------------------------------------------------------
//  Getters and setters
//------------------------------------------------------------------------------------

/** If true, fractal is limited to 2 dimensions */
- (BOOL)isFlat { return isFlat; }
- (void)setIsFlat:(BOOL) newIsFlat { isFlat = newIsFlat; }

/** Recursive depth of the fractal */
- (int)iterations { return iterations; }
- (void)setIterations:(int)newIterations { iterations = newIterations; }

/** Number of branches, a.k.a. complexity */
- (int)branchCount { return branchCount; }
- (void)setBranchCount:(int) newBranchCount { branchCount = newBranchCount; }

/** speed of updates */
- (float)speed { return speed; } 
- (void)setSpeed:(float) newSpeed { 
  int dim;
  speed = newSpeed; 

  for (dim=0; dim<3;dim++) {            
     maxDeltaV.translation[dim]	=  0.00001 * speed;
     maxDeltaV.rotation[dim]	=  0.000001 * speed;
     maxDeltaV.scale[dim]	=  0.000001 * speed;
  }
}


-(NSObject <Shape> *)shape {  return shape;  }
-(void)setShape:(NSObject <Shape> *)newShape {
    if (shape) {
        [shape release];
    }
    shape = [newShape retain]; 
}

//------------------------------------------------------------------------------------
//  Clean-up
//------------------------------------------------------------------------------------
- (void)dealloc
{
    [shape release];
    [super dealloc];
}


@end
