//
//  IFS3DDefaults.m
//  IFS3D
//
//  Copyright (c) 2001 David Leppik.
//

// Back-end handling of the defaults.  Bigger and kludgier than it needs to be.


#import <ScreenSaver/ScreenSaver.h>

#import "IFS3DDefaults.h"

#import "IFS3D.h"

#import "ColorTriangle.h"
#import "PointShape.h"
#import "Raindrop.h"
#import "RedBlueStereoPoint.h"
#import "Sphere.h"
#import "StripedPointShape.h"


@implementation IFS3DDefaults

static IFS3DDefaults *sharedInstance = nil;

static NSString *moduleName = @"net.leppik.IFS3D";

static NSDictionary * factoryDefaults = nil;



+ (id)sharedInstance {
    if (!sharedInstance) {
        sharedInstance = [[[self class] alloc] init];
    }
    return sharedInstance;
}

- (void)updateFractal {
  [fractal setIsFlat:[self boolForKey:@"fractal.isFlat"]];
  [fractal setIterations:[self intForKey:@"fractal.iterations"]];
  [fractal setBranchCount:[self intForKey:@"fractal.branchCount"]];
  [fractal setSpeed:[self floatForKey:@"fractal.speed"]];
  [fractal setShape:[IFS3DDefaults shapeForName:(NSString *)[self objectForKey:@"fractal.shape"]]];
}

- (void)useFactoryDefaults {
    if (defaults) {
      [defaults release];
    }
    defaults = [[NSMutableDictionary dictionaryWithDictionary:[IFS3DDefaults factoryDefaults]] retain];
    if ( ! fractal) {
      fractal = [[IFS3D alloc] init];
    }
    [self updateFractal];
}

-(void)load {
  if (defaults) {
    [defaults release];
  }
  defaults= [[NSMutableDictionary dictionaryWithDictionary:
                [[ScreenSaverDefaults defaultsForModuleWithName:moduleName] dictionaryRepresentation]] retain];
  // The second test is a simple check for obviously corrupt preferences
  if (defaults == nil || [defaults objectForKey:@"fractal.isFlat"] == nil) {
    [self useFactoryDefaults];
  }
  else {
    if ( ! fractal) {
      fractal = [[IFS3D alloc] init];
    }
    [self updateFractal];
  }
}

-(id)init {
  self = [super init];
  if (self) {
    [self load];
  }
  return self;
}

- (BOOL)save {
  NSUserDefaults * savedDefaults = [ScreenSaverDefaults defaultsForModuleWithName:moduleName];
  NSEnumerator * keyEnumerator = [defaults keyEnumerator];
  id key;
  while (key = [keyEnumerator nextObject]) {
    [savedDefaults setObject:[defaults objectForKey:key] forKey:key];
  }
  return [[ScreenSaverDefaults defaultsForModuleWithName:moduleName] synchronize];
}


- (id)objectForKey:(id)key {
    return [defaults objectForKey:key];
}

- (void)setObject:(id)value forKey:(id)key {
    [defaults setObject:value forKey:key];
    [self updateFractal];
}

- (BOOL)boolForKey:(id)key {
    id obj = [self objectForKey:key];
    if(obj != nil && [obj isKindOfClass:[NSNumber class]]) {
        return [(NSNumber *) obj boolValue];
    }
    else {
        if (obj != nil) {
            NSLog(@"IFS3D: Unexpected value of class %@ for preference @%", [obj class], key);
        }
        obj = [[IFS3DDefaults factoryDefaults] objectForKey:key];
        if(obj != nil && [obj isKindOfClass:[NSNumber class]]) {
            NSLog(@"IFS3D: Bad preference key @%", key);
            return NO;
        }
        return [(NSNumber *) obj boolValue];
    }
}

- (void)setBool:(BOOL)value forKey:(id)key {
    [self setObject:[NSNumber numberWithBool:value] forKey:key];
}


- (int)intForKey:(id)key {
    id obj = [self objectForKey:key];
    if(obj != nil && [obj isKindOfClass:[NSNumber class]]) {
        return [(NSNumber *) obj intValue];
    }
    else {
        if (obj != nil) {
            NSLog(@"IFS3D: Unexpected value of class %@ for preference @%", [obj class], key);
        }
        obj = [[IFS3DDefaults factoryDefaults] objectForKey:key];
        if(obj != nil && [obj isKindOfClass:[NSNumber class]]) {
            return [(NSNumber *) obj intValue];
        }
        // else failure with factory defaults
        NSLog(@"IFS3D: Bad preference key @%", key);
        return 0;
    }
}

- (void)setInt:(int)value forKey:(id)key {
    [self setObject:[NSNumber numberWithInt:value] forKey:key];
}

- (float)floatForKey:(id)key {
    id obj = [self objectForKey:key];
    if(obj != nil && [obj isKindOfClass:[NSNumber class]]) {
        return [(NSNumber *) obj floatValue];
    }
    else {
        if (obj != nil) {
            NSLog(@"IFS3D: Unexpected value of class %@ for preference @%", [obj class], key);
        }
        obj = [[IFS3DDefaults factoryDefaults] objectForKey:key];
        if(obj != nil && [obj isKindOfClass:[NSNumber class]]) {
            return [(NSNumber *) obj floatValue];
        }
        // else failure with factory defaults
        NSLog(@"IFS3D: Bad preference key @%", key);
        return 0;
    }
}

- (void)setFloat:(float)value forKey:(id)key {
    [self setObject:[NSNumber numberWithFloat:value] forKey:key];
}


+(id <Shape>)shapeForName:(NSString *)name {
    if ([name isEqualToString:@"ColorTriangle"]) {
        return [[[ColorTriangle alloc] init] autorelease];
    }
    else if ([name isEqualToString:@"PointShape"]) {
        return [[[PointShape alloc] init] autorelease];
    }
    else if ([name isEqualToString:@"Raindrop"]) {
        return [[[Raindrop alloc] init] autorelease];
    }
    else if ([name isEqualToString:@"RedBlueStereoPoint"]) {
        return [[[RedBlueStereoPoint alloc] init] autorelease];
    }
    else if ([name isEqualToString:@"Sphere"]) {
        return [[[Sphere alloc] init] autorelease];
    }
    else if ([name isEqualToString:@"StripedPointShape"]) {
        return [[[StripedPointShape alloc] init] autorelease];
    }
    // Default
    NSLog(@"IFS3D: Unknown shape @%", name);
    return [[[PointShape alloc] init] autorelease];
}

+(NSDictionary *)factoryDefaults {
    if (! factoryDefaults) {
        NSMutableDictionary *md = [[NSMutableDictionary alloc] init];
        [md setObject:[NSNumber numberWithBool:NO]	forKey:@"fractal.isFlat"];
        [md setObject:[NSNumber numberWithInt:11]	forKey:@"fractal.iterations"];
        [md setObject:[NSNumber numberWithInt:2]	forKey:@"fractal.branchCount"];
        [md setObject:@"ColorTriangle"			forKey:@"fractal.shape"];
	[md setObject:[NSNumber numberWithFloat:500.0]	forKey:@"view.rotationSpeed"];
	[md setObject:[NSNumber numberWithInt:0]	forKey:@"view.reflections"];
	[md setObject:[NSNumber numberWithFloat:450.0]	forKey:@"fractal.speed"];
        [md setObject:[NSNumber numberWithFloat:5.0]	forKey:@"view.eyeSeparation"];
        [md setObject:[NSNumber numberWithBool:NO] forKey:@"fractal.largePoint"];
        factoryDefaults = [[NSDictionary dictionaryWithDictionary:md] retain];
        [md release];
    }
    return factoryDefaults;
}

-(IFS3D *)fractal {
    return fractal;
}

//------------------------------------------------------------------------------------
//  Clean-up
//------------------------------------------------------------------------------------
- (void)dealloc
{
    [fractal release];
    [defaults release];
    [factoryDefaults release];
    [super dealloc];
}

@end
