//
//  IFS3DPreferences.m
//  IFS3D
//
//  Copyright (c) 2001 David Leppik.
//

// This controls the preferences panel.  It doesn't handle the defaults.


#import <ScreenSaver/ScreenSaver.h>

#import "IFS3DPreferences.h"

#import "IFS3D.h"

@implementation IFS3DPreferences

static IFS3DPreferences *sharedInstance = nil;
static IFS3DDefaults *defaults = nil;

+ (id)sharedInstance {
    if (!sharedInstance) {
        sharedInstance = [[[self class] alloc] init];
    }
    return sharedInstance;
}

- (id)init {
    self = [super initWithWindowNibName:@"Preferences"];

    if (self) {
        defaults = [IFS3DDefaults sharedInstance];
    }
    return self;
}

- (void)windowDidLoad {
    NSString *shapeString = (NSString *) [defaults objectForKey:@"fractal.shape"];
    
    [separationSlider setEnabled:NO];
     
    if ([shapeString isEqualToString:@"PointShape"]) {
        [shapeMatrix selectCellAtRow:0 column:0];
    }
    else if ([shapeString isEqualToString:@"StripedPointShape"]) {
        [shapeMatrix selectCellAtRow:1 column:0];
    }
    else if ([shapeString isEqualToString:@"Raindrop"]) {
        [shapeMatrix selectCellAtRow:2 column:0];
    }
    else if ([shapeString isEqualToString:@"ColorTriangle"]) {
        [shapeMatrix selectCellAtRow:3 column:0];
    } 
    else if ([shapeString isEqualToString:@"RedBlueStereoPoint"]) {
        [shapeMatrix selectCellAtRow:4 column:0];
        [separationSlider setEnabled:YES];
    }
    else if ([shapeString isEqualToString:@"Sphere"]) {
        [shapeMatrix selectCellAtRow:5 column:0];
    }
    else {
        NSLog(@"IDS3D: Unknown shape class %@ ", shapeString);
    }


    [iterationsSlider setIntValue:[defaults intForKey:@"fractal.iterations"]];
    [complexitySlider setIntValue:[defaults intForKey:@"fractal.branchCount"]];
    [self refreshShapeCount];
    
    [rotationSpeedSlider setFloatValue:[defaults floatForKey:@"view.rotationSpeed"]];
    [fractalSpeedSlider setFloatValue:[defaults floatForKey:@"fractal.speed"]];
    [separationSlider setFloatValue:[defaults floatForKey:@"view.eyeSeparation"]];
    [flatCheckbox setState:[defaults intForKey:@"fractal.isFlat"]];
    [largePointCheckbox setState:[defaults intForKey:@"fractal.largePoint"]];
    [reflectionsPopUpButton selectItemAtIndex:[defaults intForKey:@"view.reflections"]];
    [reflectionsPopUpButton synchronizeTitleAndSelectedItem];
}

- (IBAction)iterationsSliderAction:(id)sender {
    if ([self refreshShapeCount]) {
        [defaults setInt:[sender intValue] forKey:@"fractal.iterations"];
    }
    else {
        [sender setIntValue:[defaults intForKey:@"fractal.iterations"]];
        [self refreshShapeCount];
    }
}
- (IBAction)complexitySliderAction:(id)sender {
    if ([self refreshShapeCount]) {
        [defaults setInt:[sender intValue] forKey:@"fractal.branchCount"];
    }
    else {
        [sender setIntValue:[defaults intForKey:@"fractal.branchCount"]];
        [self refreshShapeCount];
    }
}

- (BOOL)refreshShapeCount {
    int i;
    int iterations = [iterationsSlider intValue];
    int complexity = [complexitySlider intValue];
    int count = pow(complexity,iterations);
    if (count == INT_MAX) {
        [shapeCountTextField setStringValue:@"too many"];
        [stepsCountTextField setStringValue:@"too many"];
        return NO;
    }
    else {
        [shapeCountTextField setIntValue:count];
        for (i=iterations-1; i>0; i--) {
            count += pow(complexity,i);
        }
        if (count == INT_MAX) {
            [stepsCountTextField setStringValue:@"too many"];
            return NO;
        }
        else {
            [stepsCountTextField setIntValue:count];
        }
    }
    return YES;
}

- (IBAction)shapeSelection:(id)sender {
    int val = [sender selectedRow];
    NSString *shapeName = (NSString *)[defaults objectForKey:@"fractal.shape"];
    
    switch (val) {
     case 0: // PointShape
        shapeName = @"PointShape";
        break;
     case 1: // StripedPointShape
        shapeName = @"StripedPointShape";
        break;
     case 2: // Raindrop
        shapeName = @"Raindrop";
        [flatCheckbox setState:YES];
        [defaults setBool:YES forKey:@"fractal.isFlat"];
        break;
     case 3: // ColorTriangle
        shapeName = @"ColorTriangle";
        [flatCheckbox setState:NO];
        [defaults setBool:NO forKey:@"fractal.isFlat"];
        break;
     case 4: // RedBlueStereoPoint
        shapeName = @"RedBlueStereoPoint";
        break;
     case 5: // Sphere
        shapeName = @"Sphere";
        [flatCheckbox setState:NO];
        [defaults setBool:NO forKey:@"fractal.isFlat"];
        break;
     default:
        break;
    }
    [defaults setObject:shapeName forKey:@"fractal.shape"];
    [separationSlider setEnabled:[[[defaults fractal] shape] isStereo]];
}

- (IBAction)flatCheckboxAction:(id)sender {
    BOOL val = [sender state];
    [defaults setBool:val forKey:@"fractal.isFlat"];
}

- (IBAction)largePointCheckboxAction:(id)sender {
    BOOL val = [sender state];
    [defaults setBool:val forKey:@"fractal.largePoint"];
}

- (IBAction)okButtonAction:(id)sender {
    [[self window] makeFirstResponder:[self window]];
    [NSApp endSheet:[self window] returnCode:NSOKButton];
    if  ( ! [defaults save] ) {
        NSLog(@"IFS3D: Could not save preferences");
        NSRunAlertPanel(@"Error", @"Could not save",@"OK",nil, nil);
    }
}

- (IBAction)revertButtonAction:(id)sender {
    [defaults load];
    [self windowDidLoad];
}

- (IBAction)resetButtonAction:(id)sender {
    [defaults useFactoryDefaults];
    [self windowDidLoad];
}


- (IBAction)rotationSpeedAction:(id)sender {
  [defaults setFloat:[sender floatValue] forKey:@"view.rotationSpeed"];
}

- (IBAction)fractalSpeedAction:(id)sender {
   [defaults setFloat:[sender floatValue] forKey:@"fractal.speed"];
}



- (IBAction)separationAction:(id)sender {
  [defaults setFloat:[sender floatValue] forKey:@"view.eyeSeparation"];
}

- (IBAction)reflectionsAction:(id)sender {
  [defaults setInt:[sender indexOfSelectedItem] forKey:@"view.reflections"];
  [reflectionsPopUpButton synchronizeTitleAndSelectedItem];
}


- (void)dealloc {
    if ([self isWindowLoaded]) {
        [[self window] setDelegate:nil];
    }

    [super dealloc];
}

@end
