//
//  RedBlueStereoPoint.m
//  IFS3D
//
//  Copyright (c) 2001 David Leppik.
//

#import "RedBlueStereoPoint.h"
#import "IFS3DDefaults.h"

@implementation RedBlueStereoPoint

/** If YES, this shape has a left-eye and right-eye mode */
- (BOOL)isStereo { return YES; }

/** IFS3D_LEFT or IFS3D_RIGHT */
- (void)setEye:(enum ViewerEye)newEye { eye = newEye; }
- (enum ViewerEye)getEye { return eye; }

/** Call callGLParams and set view-specific GL settings */
- (void)setView:(id)view { [self callGlParams]; }

/** Enable or disable render settings for the shape */
- (void)callGlParams {
    glDisable(GL_DEPTH_TEST);
    
    glEnable(GL_BLEND);
    glBlendFunc(GL_ONE,GL_ONE);
    glDisable(GL_LINE_SMOOTH);
    glDisable(GL_POINT_SMOOTH);
    glDisable(GL_POLYGON_SMOOTH);
    
    glDisable(GL_CULL_FACE);
    glDisable(GL_LIGHTING);
}

/** Set color, pen width, etc. changes before drawing. */
- (void)prepareToDraw {
    if ([[IFS3DDefaults sharedInstance] boolForKey:@"fractal.largePoint"]) {
        glPointSize(LARGE_POINT_SIZE);
    }
    else {
        glPointSize(1);
    }
    switch (eye) {
    case LEFT:
        glColor4f( 0, 0, 1, 1 );
        break;
    case RIGHT:
        glColor4f( 1, 0, 0, 1 );
        break;
    case NEITHER:
    default:
        glColor4f( 1, 1, 1, 1 );
        break;
    }
}


- (void)draw3D {
    glBegin( GL_POINTS );
        glVertex3f( 0.0, 0.0, 0.0 );
    glEnd();
}
@end
